/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Copyright (C) 1999-2004, Synaptika S.r.l.
 *
 * Massimo Vellucci <massimo.vellucci@synaptika.it>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/lock.h>
#include <asterisk/frame.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/module.h>
#include <asterisk/channel_pvt.h>
#include <asterisk/pbx.h>
#include <asterisk/cli.h>
#include <asterisk/utils.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

AST_MUTEX_DEFINE_STATIC(usecnt_lock);

static char *type = "Void";
static char *desc = "Run extension from CLI";

static int usecnt = 0;

static int void_hangup(struct ast_channel *c)
{
	ast_mutex_lock(&usecnt_lock);
	usecnt--;
	ast_mutex_unlock(&usecnt_lock);
	
	return 0;
}

static int extension_exec(char *context)
{
	int res = 0;
	struct ast_channel *tmp;
	
	if ((context == NULL) || ast_strlen_zero(context))
		return -1;
	
	tmp = ast_channel_alloc(1);
	if (tmp) {
		snprintf(tmp->name, sizeof(tmp->name), "VOID/%04x", rand());
		tmp->type = type;
		tmp->amaflags = 0;			/* Don't write to CDR */
		tmp->pvt->hangup = void_hangup;

		/* The extension is executed directly in 'h' */
		strncpy(tmp->context, context, sizeof(tmp->context) - 1);

		ast_mutex_lock(&usecnt_lock);
		usecnt++;
		ast_mutex_unlock(&usecnt_lock);
		ast_update_use_count();

		/* Set the channel in hangup status */
		ast_softhangup(tmp, AST_SOFTHANGUP_SHUTDOWN);

		/* Start PBX */
		if (ast_pbx_start(tmp)) {
			ast_log(LOG_WARNING, "Unable to start PBX on %s\n", tmp->name);
			ast_hangup(tmp);
			res = -1;
		}
	}
	
	return res;
}

static int console_run(int fd, int argc, char *argv[])
{
	if (argc != 3)
		return RESULT_SHOWUSAGE;

	extension_exec(argv[2]);
	return RESULT_SUCCESS;
}

static char run_usage[] =
"Usage: run extension <context>\n"
"       Run a given context from CLI\n";

static struct ast_cli_entry mycli = { { "run", "extension", NULL }, console_run, "Run an extension from CLI", run_usage };

int load_module()
{
	ast_cli_register(&mycli);
	return 0;
}

int unload_module()
{
	ast_cli_unregister(&mycli);
	return 0;
}

char *description()
{
	return desc;
}

int usecount()
{
	int res;
	ast_mutex_lock(&usecnt_lock);
	res = usecnt;
	ast_mutex_unlock(&usecnt_lock);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
